﻿Imports Windows.Graphics.Imaging
Imports Windows.Media
Imports Windows.Media.Editing
Imports Windows.Storage
Imports Windows.Storage.Streams

Public Class CreateBytestreams

    Public Shared Async Function GetBytestream(Picturefile As StorageFile, TargetWidth As Integer, TargetHeight As Integer) As Task(Of Pixel(,))
        'Look at:
        'http://stackoverflow.com/questions/34843058/bitmapimage-to-byte-array-and-vice-versa
        Dim Picturedata(TargetWidth - 1, TargetHeight - 1) As Pixel
        Using stream As IRandomAccessStream = Await Picturefile.OpenReadAsync
            Dim decoder As BitmapDecoder = Await BitmapDecoder.CreateAsync(stream)
            Dim transform As New BitmapTransform With {.ScaledHeight = TargetHeight, .ScaledWidth = TargetWidth}
            Dim pixeldata As PixelDataProvider = Await decoder.GetPixelDataAsync(BitmapPixelFormat.Bgra8,
                                                                                                 BitmapAlphaMode.Straight,
                                                                                                 transform,
                                                                                                 ExifOrientationMode.IgnoreExifOrientation,
                                                                                                 ColorManagementMode.DoNotColorManage)

            Dim rawpixel = pixeldata.DetachPixelData
            'Dim width = decoder.PixelWidth
            'Dim height = decoder.PixelHeight

            If rawpixel.Length = TargetWidth * TargetHeight * 4 Then
                Dim index As Integer = 0
                For y As Integer = 0 To Picturedata.GetUpperBound(1)
                    For x As Integer = 0 To Picturedata.GetUpperBound(0)
                        Picturedata(x, y) = New Pixel(rawpixel(index + 2), rawpixel(index + 1), rawpixel(index))
                        Picturedata(x, y).SetBrightness(rawpixel(index + 3))
                        index += 4
                    Next
                Next
            End If
        End Using

        Return Picturedata
    End Function
    Public Shared Async Function GetVideoBytestream(Videofile As StorageFile, FramesPerSecond As Integer, TargetWidth As Integer, TargetHeight As Integer) As Task(Of List(Of Pixel(,)))

        Dim mediaClip = Await Windows.Media.Editing.MediaClip.CreateFromFileAsync(Videofile)
        Dim mediaComposition = New MediaComposition()
        mediaComposition.Clips.Add(mediaClip)

        Dim datacontainer As New List(Of Pixel(,))
        Dim deltaTimestamp = TimeSpan.FromMilliseconds(1000 / FramesPerSecond)
        Dim currentTimestamp = New TimeSpan(0)
        While currentTimestamp < mediaComposition.Duration

            Dim thumbnail = Await mediaComposition.GetThumbnailAsync(deltaTimestamp, TargetWidth, TargetHeight, VideoFramePrecision.NearestKeyFrame)

            thumbnail.Seek(0)
            Dim decoder = Await BitmapDecoder.CreateAsync(thumbnail)
            Dim transform As New BitmapTransform With {.ScaledHeight = TargetHeight, .ScaledWidth = TargetWidth}
            Dim pixeldata As PixelDataProvider = Await decoder.GetPixelDataAsync(BitmapPixelFormat.Bgra8,
                                                                                                 BitmapAlphaMode.Straight,
                                                                                                 transform,
                                                                                                 ExifOrientationMode.IgnoreExifOrientation,
                                                                                                 ColorManagementMode.DoNotColorManage)


            '            Dim pixeldata As PixelDataProvider = Await decoder.GetPixelDataAsync

            Dim rawdata As Byte() = pixeldata.DetachPixelData

            Dim bytestream(TargetWidth - 1, TargetHeight - 1) As Pixel
            Dim index As Integer = 0
            For y As Integer = 0 To bytestream.GetUpperBound(1)
                For x As Integer = 0 To bytestream.GetUpperBound(0)
                    bytestream(x, y) = New Pixel(rawdata(index + 2), rawdata(index + 1), rawdata(index))
                    bytestream(x, y).SetBrightness(rawdata(index + 3))
                    index += 4
                Next
            Next

            datacontainer.Add(bytestream)
            currentTimestamp += (deltaTimestamp)
        End While

        Return datacontainer
    End Function



End Class
