﻿Imports Windows.ApplicationModel.Core
Imports Windows.Devices.Enumeration
Imports Windows.Devices.Portable
Imports Windows.Storage
Imports Windows.UI.Core

Public Class Devicehandler : Inherits BaseModel
    Private Shared watcher As DeviceWatcher = Nothing
    Private _count As Integer
    Public Property count As Integer
        Get
            Return _count
        End Get
        Set(value As Integer)
            _count = value
            NotifyPropertyChanged()
        End Set
    End Property
    Public Property Interfaces As New ObservableCollection(Of DeviceInformation)()

    Public Shared isEnumerationComplete As Boolean = False
    Public Shared StopStatus As String = Nothing

    Private _watcherStatus As String
    Private updateRunning As Boolean

    Public Property WatcherStatus() As String
        Get
            Return _watcherStatus
        End Get
        Set(ByVal value As String)
            _watcherStatus = value
            NotifyPropertyChanged()
        End Set
    End Property

    Public Sub WatchDevices()
        Try
            watcher = DeviceInformation.CreateWatcher()
            ' Add event handlers
            AddHandler watcher.Added, AddressOf watcher_Added
            AddHandler watcher.Removed, AddressOf watcher_Removed
            AddHandler watcher.Updated, AddressOf watcher_Updated
            AddHandler watcher.EnumerationCompleted, AddressOf watcher_EnumerationCompleted
            AddHandler watcher.Stopped, AddressOf watcher_Stopped
            watcher.Start()

            '    Await CoreApplication.MainView.CoreWindow.Dispatcher.RunAsync(
            '   CoreDispatcherPriority.Normal, Sub()
            Debug.WriteLine("Enumeration started.")
            '                                     End Sub)
        Catch generatedExceptionName As ArgumentException
            'The ArgumentException gets thrown by FindAllAsync when the GUID isn't formatted properly
            'The only reason we're catching it here is because the user is allowed to enter GUIDs without validation
            'In normal usage of the API, this exception handling probably wouldn't be necessary when using known-good GUIDs 
            Debug.WriteLine("Caught ArgumentException. Failed to create watcher.")

        End Try
    End Sub

    Private Sub StopWatcher(sender As Object, eventArgs As RoutedEventArgs)
        Try
            If watcher.Status = Windows.Devices.Enumeration.DeviceWatcherStatus.Stopped Then
                StopStatus = "The enumeration is already stopped."
            Else
                watcher.Stop()
            End If
        Catch generatedExceptionName As ArgumentException
            Debug.WriteLine("Caught ArgumentException. Failed to stop watcher.")
        End Try
    End Sub

    Private Sub watcher_Added(sender As DeviceWatcher, deviceInterface As DeviceInformation)
        Try
            Interfaces.Add(deviceInterface)
            Debug.WriteLine("Device added")

            '            Dim rootFolder = StorageDevice.FromId(deviceInterface.Id)

        Catch
        End Try
    End Sub

    Private Async Sub watcher_Updated(sender As DeviceWatcher, devUpdate As DeviceInformationUpdate)

        For i As Integer = 0 To Interfaces.Count - 1
            If Interfaces(i).Id = devUpdate.Id Then
                Interfaces(i).Update(devUpdate)
            End If
        Next
        Dim stick = Await Windows.Storage.KnownFolders.RemovableDevices.GetFoldersAsync
        For Each item In stick
            Debug.WriteLine(item.DisplayName)
        Next
        Try
            UpdateStorageDeviceFolders(stick.First)
        Catch
            Debug.WriteLine("No USB Disk found")
        End Try
    End Sub

    Private Sub watcher_Removed(sender As DeviceWatcher, devUpdate As DeviceInformationUpdate)
        For i As Integer = 0 To Interfaces.Count
            If Interfaces(i).Id = devUpdate.Id Then
                Interfaces.RemoveAt(i)
                Exit Sub
            End If
        Next
    End Sub

    Private Async Sub watcher_EnumerationCompleted(sender As DeviceWatcher, args As Object)
        isEnumerationComplete = True
        Dim stick = Await Windows.Storage.KnownFolders.RemovableDevices.GetFoldersAsync
        For Each item In stick
            Debug.WriteLine(item.DisplayName)
        Next
        If stick.Count > 0 Then
            UpdateStorageDeviceFolders(stick.First)
        End If
    End Sub

    Private Sub watcher_Stopped(sender As DeviceWatcher, args As Object)
        If watcher.Status = Windows.Devices.Enumeration.DeviceWatcherStatus.Aborted Then
            StopStatus = "Enumeration stopped unexpectedly. Click Watch to restart enumeration."
        ElseIf watcher.Status = Windows.Devices.Enumeration.DeviceWatcherStatus.Stopped Then
            StopStatus = "You requested to stop the enumeration. Click Watch to restart enumeration."
        End If
    End Sub

    Public Event AddImage(image As ImageData)
    Public Event AddVideo(video As VideoData)
    Public Event AddFont(font As Font)
    Public Event ResetCollections()

    Private Async Sub UpdateStorageDeviceFolders(StorageDevice As StorageFolder)


        Try
            If Not updateRunning Then
                updateRunning = True

                RaiseEvent ResetCollections()


                'Dim root = KnownFolders.RemovableDevices

                'Dim devices = Await root.GetFoldersAsync

                'For Each device In StorageDevice.get 'devices
                '    Debug.WriteLine("")
                '    Debug.WriteLine(String.Format("Driveletter: {0}", device.Name))
                '    Debug.WriteLine("")
                '    Debug.WriteLine("Folders")
                Dim folders = Await StorageDevice.GetFoldersAsync ' device.GetFoldersAsync
                For Each item In folders
                        Debug.WriteLine(item.Name)
                    Next
                    Debug.WriteLine("")
                    Debug.WriteLine("Files")
                Dim files = Await StorageDevice.GetFilesAsync ' device.GetFilesAsync
                For Each item In files
                        Debug.WriteLine(item.Name)

                        Select Case item.DisplayType
                            Case "image"

                                Await Task.Run(Sub()
                                                   Dim newImage = New ImageData(item)
                                                   RaiseEvent AddImage(newImage)
                                                   Debug.WriteLine("Image added")
                                               End Sub)

                            Case "video"
                            Try
                                Await Task.Delay(10000)
                                Dim newVideo = New VideoData(item)
                                RaiseEvent AddVideo(newVideo)
                                Debug.WriteLine("Video added")
                            Catch
                                Dim ddd = 0
                            End Try


                        Case "font"
                                Dim newFont = New Font(item)

                                RaiseEvent AddFont(newFont)
                                Debug.WriteLine("Font added")

                        End Select
                    Next

                '  Next
                Dim abc = 0

                'Catch ex As Exception
                '    Debug.WriteLine("")
                '    Debug.WriteLine("Error catching files:")
                '    Debug.WriteLine(ex.Message)
                '    Debug.WriteLine("")
                'End Try
                Debug.WriteLine("")
                Debug.WriteLine("All Files processed")
                Debug.WriteLine("")

                Debug.WriteLine("Files processed")

                RaiseEvent DeviceEnumerationDone()
            End If
        Catch ex As Exception
            Dim abc = 0
        End Try
        updateRunning = False
    End Sub

    Public Event DeviceEnumerationDone()
End Class
'        Private dispatcher As Windows.UI.Core.CoreDispatcher
