﻿Imports System.Text
Imports System.Threading
Imports Windows.ApplicationModel.Core
Imports Windows.Devices.Enumeration
Imports Windows.Devices.SerialCommunication
Imports Windows.Storage
Imports Windows.Storage.Streams
Imports Windows.System.Threading
Imports Windows.UI
Imports Windows.UI.Core

Public Class Displayconnection : Inherits BaseModel

    Public Event DisplayTestDone()

    Private _Connected As Boolean = False
    Public Property Connected() As Boolean
        Get
            Return _Connected
        End Get
        Set(ByVal value As Boolean)
            _Connected = value
            NotifyPropertyChanged()
        End Set
    End Property

    Private SerialPort As SerialDevice
    Public buffer(143, 39) As Pixel
    Private ApplicationData
    Public Sub New()

        ApplicationData = Windows.Storage.ApplicationData.Current
        localSettings = ApplicationData.LocalSettings
        Try
            _Brightness = localSettings.Values("Brightness")
        Catch
            _Brightness = &HFF
            localSettings.Values("Brightness") = _Brightness
        End Try

        For y As Integer = 0 To buffer.GetUpperBound(1)
            For x As Integer = 0 To buffer.GetUpperBound(0)
                buffer(x, y) = New Pixel(0, 0, 0)
                buffer(x, y).SetBrightness(_Brightness)
            Next
        Next

        Serial()

    End Sub

    Private TestLoopCounter As Integer = 0
    Private tpt As ThreadPoolTimer


    Public Property cmdTestDisplay() As New RelayCommand(Sub()
                                                             TestLoopCounter = 0
                                                             tpt = ThreadPoolTimer.CreatePeriodicTimer(AddressOf DisplayTestLoop, New TimeSpan(0, 0, 2))
                                                         End Sub, True)

    Private _Brightness As Byte
    Public Property CurrentBrightness() As Integer
        Get
            Return _Brightness * 100 / &HFF
        End Get
        Set(ByVal value As Integer)

            _Brightness = CByte(value * &HFF / 100)
            NotifyPropertyChanged()
            localSettings.Values("Brightness") = _Brightness
            BrightnessChanged = True
        End Set
    End Property
    Dim BrightnessChanged As Boolean

    Private Sub DisplayTestLoop()
        TestLoopCounter += 1
        Select Case TestLoopCounter
            Case 1
                For x As Integer = 0 To buffer.GetUpperBound(0)
                    For y As Integer = 0 To buffer.GetUpperBound(1)
                        buffer(x, y) = New Pixel(0, 255, 0)
                        buffer(x, y).SetBrightness(255)
                    Next
                Next

            Case 2
                For x As Integer = 0 To buffer.GetUpperBound(0)
                    For y As Integer = 0 To buffer.GetUpperBound(1)
                        buffer(x, y) = New Pixel(255, 0, 0)
                        buffer(x, y).SetBrightness(255)
                    Next
                Next

            Case 3
                For x As Integer = 0 To buffer.GetUpperBound(0)
                    For y As Integer = 0 To buffer.GetUpperBound(1)
                        buffer(x, y) = New Pixel(255, 255, 255)
                        buffer(x, y).SetBrightness(255)
                    Next
                Next

            Case 4
                For x As Integer = 0 To buffer.GetUpperBound(0)
                    For y As Integer = 0 To buffer.GetUpperBound(1)
                        buffer(x, y) = New Pixel(0, 0, 255)
                        buffer(x, y).SetBrightness(255)
                    Next
                Next

            Case 5
                Dim val As Color
                For x As Integer = 0 To buffer.GetUpperBound(0)
                    For y As Integer = 0 To buffer.GetUpperBound(1)
                        Dim c = buffer(x, y).GetColor
                        c.A -= CByte(c.A * 50 / 100)
                        val = c
                        buffer(x, y).SetColor(c)
                        buffer(x, y).SetBrightness(255)
                    Next
                Next
                If val.A > 10 Then
                    TestLoopCounter -= 1
                End If

            Case 6
                For x As Integer = 0 To buffer.GetUpperBound(0)
                    For y As Integer = 0 To buffer.GetUpperBound(1)
                        buffer(x, y) = New Pixel(x, 0, y + 1)
                        buffer(x, y).SetBrightness(255)
                    Next
                Next

            Case 7
                For x As Integer = 0 To buffer.GetUpperBound(0)
                    For y As Integer = 0 To buffer.GetUpperBound(1)
                        buffer(x, y).SetColor(Color.FromArgb(255, 255 Mod (x + 1), 255 / (y + 1), y / 2))
                        buffer(x, y).SetBrightness(255)
                    Next
                Next

            Case Else
                tpt.Cancel()
                RaiseEvent DisplayTestDone()
        End Select
    End Sub

    Public Async Sub Serial()
        Dim aqs = SerialDevice.GetDeviceSelectorFromUsbVidPid(&H16C0, &H483)
        Dim dis = Await DeviceInformation.FindAllAsync(aqs)
        If dis.Count > 0 Then
            ' Find the serial device with our selector string  
            SerialPort = Await SerialDevice.FromIdAsync(dis(0).Id)
            ' Create an serial device with our selected device 

            ' Configure serial settings 

            SerialPort.WriteTimeout = TimeSpan.FromMilliseconds(500)
            SerialPort.ReadTimeout = TimeSpan.FromMilliseconds(500)
            SerialPort.BaudRate = 921600 ' 115200
            ' mini UART: only standard baudrates 
            SerialPort.Parity = SerialParity.None
            ' mini UART: no parities 
            SerialPort.StopBits = SerialStopBitCount.One
            ' mini UART: 1 stop bit 
            SerialPort.DataBits = 8

            Connected = True

            ' Write a string out over serial 

            'Dim txBuffer As String = "?"
            'Dim dataWriter As New DataWriter()
            'dataWriter.WriteString(txBuffer)
            'Dim bytesWritten As UInteger = Await SerialPort.OutputStream.WriteAsync(dataWriter.DetachBuffer())

            'ReDim buffer(143, 39)
            For x As Integer = 0 To buffer.GetUpperBound(0)
                For y As Integer = 0 To buffer.GetUpperBound(1)
                    buffer(x, y) = New Pixel(0, 0, y + 1)
                    buffer(x, y).SetBrightness(&HFF)
                Next
            Next
            BrightnessChanged = False
            currentSize = buffer.Length
            oldSize = buffer.Length


            SerialSendingTask.Start()

            sendTask.Start()

            Await Task.Delay(100)

            Receiveloop.Start()

            ' Read data in from the serial port 
            ' tp = ThreadPoolTimer.CreatePeriodicTimer(AddressOf send, New TimeSpan(0, 0, 1))
            '  tp2 = ThreadPoolTimer.CreatePeriodicTimer(AddressOf change, New TimeSpan(0, 0, 10))

        Else
            Debug.WriteLine("No Displaycontroller found")
            Connected = False

        End If

    End Sub

    Private Receiveloop As New Task(Async Sub()
                                        Do
                                            Try
                                                Dim res = Await ReadResponse(SerialPort)
                                                Debug.WriteLine(res)
                                                Await Task.Delay(50)
                                            Catch
                                            End Try
                                        Loop
                                    End Sub)


    Private SendingTaskTimer As ThreadPoolTimer

    Public Sub ActivateBrightness()
        BrightnessChanged = True
    End Sub
    Shared Async Function ReadResponse(device As SerialDevice) As Task(Of String)
        Dim sr As New DataReader(device.InputStream)

        If sr IsNot Nothing Then
            ' read response
            Dim response = Await sr.LoadAsync(1000)
            ' read 1000 bytes
            Dim data = sr.ReadString(response)
            sr.DetachStream()

            Return data
        End If
        Return "Error with serial"

    End Function

    Friend Shared Async Function SendCommand(device As SerialDevice, datatosend As Byte()) As Task
        Using sw As DataWriter = New DataWriter(device.OutputStream)

            ' Send Command
            sw.WriteBytes(datatosend)
            '            Debug.WriteLine(Convert.ToString("Sending data: ") & datatosend.Length)
            Await sw.StoreAsync()
            sw.DetachStream()
        End Using
    End Function


    '  Dim tp2 As ThreadPoolTimer

    Dim currentSize As Integer
    Dim oldSize As Integer

    Dim SendBuffer(144 * 40 * 3 - 1) As Byte

    Private sendTask As New Task(Sub()

                                     Do

                                         Try

                                             If BrightnessChanged Then
                                                 For Each item In buffer
                                                     item.SetBrightness(_Brightness)
                                                 Next
                                                 BrightnessChanged = False
                                             End If


                                             Dim lines() As Integer = {1, 3, 6, 8, 11, 13, 16, 18, 21, 23, 26, 28, 31, 33, 36, 38}

                                             Dim counter As Integer = 0
                                             Dim st As New Stopwatch
                                             st.Start()
                                             For y As Integer = 0 To buffer.GetUpperBound(1)
                                                 For x As Integer = 0 To buffer.GetUpperBound(0)

                                                     If lines.Contains(y) Then

                                                         Dim xVal = (buffer.GetUpperBound(0)) - x
                                                         Dim color = buffer(xVal, y).GetColorRaw
                                                         SendBuffer(counter) = color(0) : counter += 1
                                                         SendBuffer(counter) = color(1) : counter += 1
                                                         SendBuffer(counter) = color(2) : counter += 1
                                                     Else
                                                         Dim color = buffer(x, y).GetColorRaw
                                                         SendBuffer(counter) = color(0) : counter += 1
                                                         SendBuffer(counter) = color(1) : counter += 1
                                                         SendBuffer(counter) = color(2) : counter += 1
                                                     End If
                                                 Next
                                             Next
                                             '   Await SendCommand(SerialPort, SendBuffer)
                                             st.Stop()
                                             '      Debug.WriteLine("Transfertime: {0}, Size: {1}", st.ElapsedMilliseconds, SendBuffer.Length)
                                         Catch ex As Exception
                                             Debug.WriteLine("Byte reorder error")
                                         End Try

                                     Loop

                                 End Sub)

    Private SerialSendingTask As New Task(Async Sub()
                                              Do
                                                  Try
                                                      Await SendCommand(SerialPort, SendBuffer)
                                                      Await Task.Delay(5)
                                                  Catch
                                                      Debug.WriteLine("Sending error")
                                                  End Try
                                              Loop
                                          End Sub, TaskCreationOptions.LongRunning Or TaskCreationOptions.RunContinuationsAsynchronously)


    Private localSettings As ApplicationDataContainer
End Class
